image.symbolic-circular {
  background-color: color-mix(in srgb, currentColor 8%, transparent);
  min-width: 32px;
  min-height: 32px;
  border-radius: 50%;
}

.error-box image,
.error-box label {
  color: var(--error-color);
}

.error-box label {
  font-weight: 600;
}
