.drag-handle {
  color: color-mix(in srgb, var(--window-fg-color) 40%, transparent);
}

.drag-handle:backdrop {
  color: color-mix(in srgb, var(--window-fg-color) 40%, transparent);
}

row.illustrated picture:disabled {
  /*
   * FIXME: This is a workaround to handle disabled pictures
   * see: https://gitlab.gnome.org/GNOME/libadwaita/-/merge_requests/814
   */
  opacity: 0.5;
}

row.illustrated .frame {
  border-radius: 6px;
}

.cc-placeholder-row {
  padding: 18px;
}
