.security-level-icon {
  border-radius: 9999px;
  min-width: 64px;
  min-height: 64px;
}

.security-level-icon {
  border-radius: 9999px;
  min-width: 64px;
  min-height: 64px;
}

.security-level-icon.good {
  color: var(--success-color);
  background-color: color-mix(in srgb, var(--success-color) 25%, transparent);
}

.security-level-icon.error {
  color: var(--error-color);
  background-color: color-mix(in srgb, var(--error-color) 25%, transparent);
}

.security-level-icon.warning {
  color: var(--warning-color);
  background-color: color-mix(in srgb, var(--warning-color) 25%, transparent);
}

.security-level-icon.neutral {
  color: color-mix(in srgb, currentColor 40%, transparent);
  background-color: color-mix(in srgb, currentColor 40%, transparent);
}

image.success-icon {
  color: var(--success-color);
}

image.error-icon {
  color: var(--error-color);
}

row.hide-arrow image.expander-row-arrow {
  opacity: 0;
}
