/* Battery levels */

levelbar block.warning-battery-offset {
  background-color: var(--error-color);
  border-color: var(--error-color);
}

levelbar block.low-battery-offset {
  background-color: var(--warning-color);
  border-color: var(--warning-color);
}

levelbar block.high-battery-offset {
  background-color: var(--success-color);
  border-color: var(--success-color);
}

/* Power profiles */

.power-profile.low-power {
  color: var(--success-color);
}

.power-profile.performance {
  color: var(--error-color);
}

.power-profile-info-row {
  padding: 6px 0;
}
