display-arrangement {
  font-weight: bold;
  font-size: larger;
}

display-arrangement.monitor {
  border: solid 1px color-mix(in srgb, currentColor 15%, transparent);
  margin: 0px 0px 1px 1px;
  background: var(--window-bg-color);
  padding: 0.4em;
  border-radius: 5px;
}

display-arrangement.monitor.primary {
  border-top: 0.4em solid #000000;
}

.monitor-label {
  border-radius: 50%;
  font-weight: bold;
  min-width: 1.5em;
  min-height: 1.5em;
  color: #000;
  background: #ddd;
  font-feature-settings:"tnum";
}
