/*
 * SPDX-License-Identifier: GPL-3.0-or-later
 * SPDX-FileCopyrightText: 2024 GNOME Foundation, Inc.
 */

/* Break scheduling */
.break-schedule-row popover listview header {
  /* Override the default GTK theming for the header row of a listview because
   * that much top padding looks weird when the listview is the only child of a
   * popover. The original padding was 18px. */
  padding-top: 6px;
}

/* Screen time statistics */
row.screen-time-statistics-row box.main-box {
  margin: 12px 16px 12px 16px;
}

row.screen-time-statistics-row .primary-label,
row.screen-time-statistics-row .primary-value {
  color: var(--view-fg-color);
}

row.screen-time-statistics-row .primary-label {
  font-size: 100%;
}

row.screen-time-statistics-row .primary-value {
  font-weight: 800;
  font-size: 181%;
  margin-top: 6px;
  margin-left: -2px; /* offset the label to account for misalignment illusion with large font */
}

row.screen-time-statistics-row .secondary-label {
  margin-top: 6px;
  margin-bottom: 2px;
  font-size: 82%;
  text-transform: uppercase;
  color: color-mix(in srgb, var(--view-fg-color) var(--dim-opacity), transparent);
}

row.screen-time-statistics-row .secondary-value {
  color: var(--view-fg-color);
}

/* Bar chart */
bar-chart bar-group {
  padding: 0 6px;
  transition: background 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94);
  border-radius: 6px;
}

bar-chart bar-group:hover {
  background-color: color-mix(in srgb, currentColor 7%, transparent);  /* equivalent to $hover_color from libadwaita */
}

bar-chart bar {
  border-radius: 5px 5px 0 0;
  background-color: color-mix(in srgb, var(--accent-bg-color) 27%, transparent);
}

bar-chart bar:selected {
  background-color: var(--accent-bg-color);
  transition: background 400ms cubic-bezier(0.25, 0.46, 0.45, 0.94);
}

bar-chart bar-group:focus-within {
  transition-property: outline-style, outline-width, outline-offset, outline-color;
  transition-duration: 300ms;
  animation-timing-function: ease-in-out;

  outline-style: solid;
  outline-color: color-mix(in srgb, var(--accent-color) 50%, transparent);  /* equivalent to $focus_border_color from libadwaita */
  outline-width: 2px;
  outline-offset: -2px;
}

/* Chart labels */
label.continuous-axis-label {
  color: color-mix(in srgb, var(--view-fg-color) var(--dim-opacity), transparent);
  margin: 0 6px;
  font-size: 82%;
}

label.discrete-axis-label {
  margin-top: 6px;
}

