/*************************************************
 * CcContentRow, mostly copied from AdwActionRow *
 *************************************************/

row.content box.header,
row.content box.contents {
  margin-left: 12px;
  margin-right: 12px;
  border-spacing: 6px;
  min-height: 50px;
}
row.content box.title {
  margin-top: 6px;
  margin-bottom: 6px;
  border-spacing: 3px;
  padding: 0;
}
row.content .prefixes,
row.content .suffixes {
  border-spacing: 6px;
}
row.content .prefixes:dir(ltr) {
  margin-right: 6px;
}
row.content .prefixes:dir(rtl) {
  margin-left: 6px;
}

row.split-row radio {
  background-position: center;
  background-size: 14px;
}

row.split-row radio:checked {
  background-image: -gtk-icontheme("bullet-symbolic");
}

row.split-row box.card {
  border-radius: 9px;
  transition-property: box-shadow, background, outline, outline-offset;
  transition-duration: 250ms;
}

row.split-row box.card:not(:hover) {
  box-shadow: unset;
  background: unset;
}

row.split-row.content box.contents {
  margin-left: 3px;
  margin-right: 3px;
  margin-bottom: 3px;
}

row.split-row box.header {
  min-height: 0;
  margin-top: 9px;
  margin-bottom: 3px;
}
