wheel {
  background: color-mix(in srgb, currentColor 8%, transparent);
  border-radius: 100%;
  margin: 40px;
  min-width: 280px;
  min-height: 280px;
}

wheel button {
  padding: 5px;
}

wheel button.playing {
  background-color: var(--accent-bg-color);
  color: var(--accent-fg-color);
}

.test-button-bin {
  border-radius: 9999px;
  padding: 2px;
}
