.mouse-test-viewport {
    background: #26a269;
}

.mouse-test-arrow {
    color: white;
}

.top-arrow {
    padding-bottom:2em;
    background: linear-gradient(to top, transparent, rgba(0,0,0,0.5));
}
.bottom-arrow {
    padding-top:2em;
    background: linear-gradient(to bottom, transparent, rgba(0,0,0,0.5));
}

.click-status-emblem {
    border-radius: 9999px;
    background-color: color-mix(in srgb, currentColor 20%, transparent);
    color: color-mix(in srgb, currentColor 50%, transparent);
    padding: 10px;
}

.click-status-emblem.success {
    background-color: var(--success-bg-color);
    color: var(--success-fg-color);
}
