/* Increase the thickness and draw a gradient of color temperature */
.night-light-temperature trough {
    padding-top: 2px;
    padding-bottom: 2px;
    --temp-1700K-color: rgb(255 122 0); /* From colord, blackbody_data_d65plankian LUT */
    --temp-4700K-color: rgb(255 225 196); /* From colord, blackbody_data_d65plankian LUT */
    background: linear-gradient(to right, var(--temp-4700K-color), var(--temp-1700K-color));
}

.night-light-temperature:dir(rtl) trough {
    background: linear-gradient(to left, var(--temp-4700K-color), var(--temp-1700K-color));
}

.padded-spinbutton {
    min-width: 40px;
}

.unpadded-button {
    padding: 6px;
}
