/*
 * SPDX-License-Identifier: GPL-3.0-or-later
 * SPDX-FileCopyrightText: 2024 GNOME Foundation, Inc.
 */

row.screen-time-statistics-row .secondary-label {
  color: var(--view-fg-color);
}

row.screen-time-statistics-row .secondary-value {
  color: var(--view-fg-color);
}

/* Bar chart */
bar-chart bar {
  background-color: color-mix(in srgb, var(--accent-bg-color) 60%, var(--view-bg-color));
}

bar-chart bar:selected {
  background-color: var(--accent-bg-color);
}

bar-chart bar-group:focus-within {
  outline-color: color-mix(in srgb, var(--accent-color) 80%, transparent);  /* equivalent to $focus_border_color from libadwaita */
}

label.continuous-axis-label {
  color: var(--view-fg-color);
}

